<?php

/**
 * @file
 * Administration pages for configuration translations.
 */

use Drupal\Core\Language\Language;
use Drupal\Core\Config\Schema\Element;
use Drupal\user\Plugin\Core\Entity\User;
use Drupal\config_translation\ConfigGroupMapper;
use Drupal\config_translation\ConfigMapperInterface;
use Drupal\Core\Config\Config;

/**
 * Main translation overview page.
 *
 * Presents an index of all configuration elements that can be translated.
 */
function config_translation_main_overview_page() {
  $page = array();
  $header = array(t('Name'), t('Original language'), t('Operations'));
  $page['translatables'] = array(
    '#type' => 'table',
    '#header' => $header,
    '#empty' => t('No translatable configuration found.'),
  );

  $config_groups = config_translation_get_groups();
  foreach ($config_groups as $group) {
    if ($group instanceof ConfigGroupMapper && $group->hasSchema() && $group->hasTranslatable()) {
      // Figure out language code and name for this configuration. Shipped
      // configuration might not have an explicit language code and English
      // might not be present as a configured language on the site. Unless
      // the configuration has an explicit language code we assume English.
      // If the configuration has an explicit language code of none, then that
      // is not translatable.
      $langcode = $group->getLangcode();
      if ($langcode != LANGUAGE_NOT_SPECIFIED) {
        $path = $group->getBasePath();
        $language = language_load($langcode);
        $language_name = (empty($language) && $langcode == 'en') ? t('Built-in English') : language_name($langcode);

        $page['translatables'][$path]['label'] = array(
          '#markup' => $group->getTitle(),
        );
        $page['translatables'][$path]['language'] = array(
          '#markup' => $language_name,
        );
        // As long there is only one operation, use the link type as in
        // node.admin.inc for node administration.
        $page['translatables'][$path]['operations'] = array(
          '#type' => 'link',
          '#title' => t('Translate'),
          '#href' => $path . '/translate',
        );
      }
    }
  }
  return $page;
}

/**
 * Language translations overview page for a configuration name.
 *
 * @param \Drupal\config_translation\ConfigMapperInterface $mapper
 *   Configuration mapper.
 * @param mixed $path_arg
 *   Path argument from the menu system (entity id or loaded entity for
 *   configuration entities, NULL otherwise).
 *
 * @return array
 *   Page render array.
 */
function config_translation_item_overview_page(ConfigMapperInterface $mapper, $path_arg) {
  // Get configuration group for this mapper.
  $group = $mapper->getConfigGroup($path_arg);

  drupal_set_title(t('Translations for %label', array('%label' => $group->getTitle())), PASS_THROUGH);

  // It is possible the original language this configuration was saved with is
  // not on the system. For example, the configuration shipped in English but
  // the site has no English configured. Represent the original language in the
  // table even if it is not currently configured.
  $languages = language_list();
  $original_langcode = $group->getLangcode();
  if (!isset($languages[$original_langcode])) {
    $language_name = language_name($original_langcode);
    if ($original_langcode == 'en') {
      $language_name = t('Built-in English');
    }
    // Create a dummy language object for this listing only.
    $languages[$original_langcode] = new Language(array('langcode' => $original_langcode, 'name' => $language_name));
  }

  $path = $group->getBasePath();
  $header = array(t('Language'), t('Operations'));
  $page = array();
  $page['languages'] = array(
    '#type' => 'table',
    '#header' => $header,
  );
  foreach ($languages as $language) {
    if ($language->langcode == $original_langcode) {
      $page['languages'][$language->langcode]['language'] = array(
        '#markup' => '<strong>' . t('@language (original)', array('@language' => $language->name)) . '</strong>'
      );

      // @todo: the user translating might as well not have access to
      // edit the original configuration. They will get a 403 for this
      // link when clicked. Do we know better?
      $operations = array();
      $operations['edit'] = array(
        'title' => t('Edit'),
        'href' => $path,
      );
      $page['languages'][$language->langcode]['operations'] = array(
        '#type' => 'operations',
        '#links' => $operations,
      );
    }
    else {
      $page['languages'][$language->langcode]['language'] = array(
        '#markup' => $language->name,
      );
      $operations = array();

      // Check if translation exist for this language.
      if (!$group->hasTranslation($language)) {
        $operations['add'] = array(
          'title' => t('Add'),
          'href' => $path . '/translate/add/' . $language->langcode,
        );
      }
      else {
        $operations['edit'] = array(
          'title' => t('Edit'),
          'href' => $path . '/translate/edit/' . $language->langcode,
        );
        $operations['delete'] = array(
          'title' => t('Delete'),
          'href' => $path . '/translate/delete/' . $language->langcode,
        );
      }
      $page['languages'][$language->langcode]['operations'] = array(
        '#type' => 'operations',
        '#links' => $operations,
      );
    }
  }
  return $page;
}

/**
 * Configuration page wrapper for translation editing form.
 *
 * @param string $action
 *   Action identifier, either 'add' or 'edit'. Used to provide proper
 *   labeling on the screen.
 * @param \Drupal\config_translation\ConfigMapperInterface $mapper
 *   Configuration mapper.
 * @param mixed $path_arg
 *   Path argument from the menu system (entity id or loaded entity for
 *   configuration entities, NULL otherwise).
 * @param \Drupal\Core\Language\Language $language
 *   A language object.
 */
function config_translation_item_translate_page($action, ConfigMapperInterface $mapper, $path_arg, $language) {
  // Get configuration group for this mapper.
  $group = $mapper->getConfigGroup($path_arg);

  $replacement = array(
    '%label' => $group->getTitle(),
    '@language' => strtolower($language->name),
  );
  switch ($action) {
    case 'add':
      drupal_set_title(t('Add @language translation for %label', $replacement), PASS_THROUGH);
      break;
    case 'edit':
      drupal_set_title(t('Edit @language translation for %label', $replacement), PASS_THROUGH);
      break;
  }

  // Get base language configuration to display in the form before entering
  // into the language context for the form. This avoids repetitively going
  // in and out of the language context to get original values later.
  $base_config = $group->getConfigData();
  config_translation_enter_context($language);
  return drupal_get_form('config_translation_form', $group, $language, $base_config);
}

/**
 * Build configuration form with metadata and values.
 *
 * @param array $form
 *   An associative array containing the structure of the form.
 * @param array $form_state
 *   An associative array containing the current state of the form.
 * @param \Drupal\config_translation\ConfigGroupMapper $group
 *   Configuration name group.
 * @param \Drupal\Core\Language\Language $language
 *   A language object.
 * @param array $base_config
 *   An array of base language configuration data keyed by configuration names.
 *
 * @return array
 *   An associative array containing the structure of the form.
 */
function config_translation_form($form, &$form_state, ConfigGroupMapper $group, $language, $base_config = array()) {
  $form['group'] = array(
    '#type' => 'value',
    '#value' => $group,
  );
  $form['language'] = array(
    '#type' => 'value',
    '#value' => $language,
  );
  foreach ($group->getNames() as $id => $name) {
    $form[$id] = array(
      '#type' => 'container',
      '#tree' => TRUE,
    );
    $form[$id] += config_translation_build_form(config_typed()->get($name), config($name)->get(), $base_config[$name]);
  }

  $form['actions']['#type'] = 'actions';
  $form['actions']['submit'] = array('#type' => 'submit', '#value' => t('Save translation'));
  return $form;
}

/**
 * Submit handler to save config translation.
 */
function config_translation_form_submit(&$form, &$form_state) {
  $form_values = $form_state['values'];
  $language = $form_values['language'];
  $group = $form_values['group'];

  // For the form submission handling, use the override free context.
  config_context_enter('config.context.free');

  foreach ($group->getNames() as $id => $name) {
    // Set config values based on form submission and original values.
    $base_config = config($name);
    $translation_config = config('locale.config.' . $language->langcode . '.' . $name);
    config_translation_set_config($base_config, $translation_config, $form_values[$id]);

    // If no overrides, delete language specific config file.
    $saved_config = $translation_config->get();
    if (empty($saved_config)) {
      $translation_config->delete();
    }
    else {
      $translation_config->save();
    }
  }

  config_context_leave();

  drupal_set_message(t('Updated @language configuration translations successfully.', array('@language' => $language->name)));
  $form_state['redirect'] = $group->getBasePath() . '/translate';
}

/**
 * Sets configuration based on a nested form value array.
 *
 * @param \Drupal\Core\Config\Config $base_config
 *   Base language configuration values instance.
 * @param \Drupal\Core\Config\Config $translation_config
 *   Translation configuration instance. Values from $config_values will be set
 *   in this instance.
 * @param array $config_values
 *   A simple one dimensional or recursive array:
 *     - simple:
 *        array(name => 'French site name')
 *     - recursive:
 *        cancel_confirm => array(
 *          cancel_confirm.subject => 'Subject'
 *          cancel_confirm.body => 'Body content'
 *        );
 *   Either format is used, the nested arrays are just containers and not
 *   needed for saving the data.
 */
function config_translation_set_config(Config $base_config, Config $translation_config, array $config_values) {
  foreach ($config_values as $key => $value) {
    if (is_array($value)) {
      // Traverse into this level in the configuration.
      config_translation_set_config($base_config, $translation_config, $value);
    }
    else {
      // Save value, if different from original configuration. If same as
      // original configuration, remove override.
      if ($base_config->get($key) !== $value) {
        $translation_config->set($key, $value);
      }
      else {
        $translation_config->clear($key);
      }
    }
  }
}

/**
 * Formats configuration schema as a form tree.
 *
 * @param $schema
 *   Schema definition of configuration.
 * @param $config
 *   Configuration object of requested language.
 * @param $base_config
 *   Configuration object of base language.
 * @param bool $collapsed
 *   Flag to set collapsed state.
 * @param string|NULL $base_key
 *   Base configuration key.
 *
 * @return array
 *   An associative array containing the structure of the form.
 *
 * @todo
 *   Make this conditional on the translatable schema property from
 *   http://drupal.org/node/1905152 - currently hardcodes label and text.
 */
function config_translation_build_form($schema, $config, $base_config, $collapsed = FALSE, $base_key = '') {
  $build = array();
  foreach ($schema as $key => $element) {
    $element_key = implode('.', array_filter(array($base_key, $key)));
    $definition = $element->getDefinition() + array('label' => t('N/A'));
    if ($element instanceof Element) {
      // Build sub-structure and include it with a wrapper in the form
      // if there are any translatable elements there.
      $sub_build = config_translation_build_form($element, $config[$key], $base_config[$key], TRUE, $key);
      if (!empty($sub_build)) {
        $build[$key] = array(
          '#type' => 'details',
          '#title' => t($definition['label']),
          '#collapsible' => TRUE,
          '#collapsed' => $collapsed,
        ) + $sub_build;
      }
    }
    else {
      $type = $element->getType();
      switch ($type) {
        case 'label':
          $type = 'textfield';
          break;
        case 'text':
          $type = 'textarea';
          break;
        default:
          continue(2);
          break;
      }
      $value = $config[$key];
      $description = format_string('<strong>Source string</strong>: !source_string', array('!source_string' => nl2br($base_config[$key]) ?: t('(Empty)')));
      $build[$element_key] = array(
        '#type' => $type,
        '#title' => t($definition['label']),
        '#default_value' => $value,
        '#description' => $description,
      );
    }
  }
  return $build;
}

/**
 * Creates the form for confirmation of deleting a translation.
 *
 * @param array $form
 *   An associative array containing the structure of the form.
 * @param array $form_state
 *   An associative array containing the current state of the form.
 * @param \Drupal\config_translation\ConfigMapperInterface $mapper
 *   Configuration mapper.
 * @param mixed $path_arg
 *   Path argument from the menu system (entity id or loaded entity for
 *   configuration entities, NULL otherwise).
 * @param \Drupal\Core\Language\Language $language
 *   A language object.
 *
 * @return array
 *   An associative array containing the structure of the form.
 *
 * @see config_translation_item_delete_form_submit().
 * @ingroup forms
 */
function config_translation_item_delete_form($form, &$form_state, ConfigMapperInterface $mapper, $path_arg, $language) {
  // Get configuration group for this mapper.
  $group = $mapper->getConfigGroup($path_arg);

  $form['group'] = array(
    '#type' => 'value',
    '#value' => $group,
  );
  $form['language'] = array(
    '#type' => 'value',
    '#value' => $language,
  );
  return confirm_form($form,
    t('Are you sure you want to delete the @language translation of %label?', array('%label' => $group->getTitle(), '@language' => $language->name)),
    $group->getBasePath() . '/translate',
    t('This cannot be undone.'),
    t('Delete'),
    t('Cancel')
  );
}

/**
 * Form submission handler for config_translation_item_delete_form().
 *
 * @see config_translation_item_delete_form().
 */
function config_translation_item_delete_form_submit($form, &$form_state) {
  $form_values = $form_state['values'];
  $group = $form_values['group'];
  $language = $form_values['language'];

  $storage = drupal_container()->get('config.storage');
  foreach ($group->getNames() as $name) {
    $storage->delete('locale.config.' . $language->langcode . '.' . $name);
  }
  // @todo: do we need to flush caches? The config change may affect page display.
  // drupal_flush_all_caches();

  drupal_set_message(t('@language translation of %label was deleted', array('%label' => $group->getTitle(), '@language' => $language->name)));
  $form_state['redirect'] = $group->getBasePath() . '/translate';
}

/**
 * Use a dummy user to get overrides for a specific language.
 *
 * See http://drupal.org/node/1905152 for an upcoming API to access
 * translations easier through typed data.
 *
 * @param \Drupal\Core\Language\Language $language
 *   A language object.
 */
function config_translation_enter_context($language) {
  $user_config_context = config_context_enter('Drupal\user\UserConfigContext');
  $account = new User(array('preferred_langcode' => $language->langcode), 'user');
  $user_config_context->setAccount($account);
}
