<?php

/**
 * @file
 * Contains \Drupal\config_translation\ConfigEntityMapper.
 */

namespace Drupal\config_translation;

use Drupal\Core\Language\Language;
use Drupal\config_translation\ConfigGroupMapper;

/**
 * Configuration entity mapper.
 */
class ConfigEntityMapper implements ConfigMapperInterface {

  /**
   * Base path for entity editing.
   *
   * @var string
   */
  private $base_path = '';

  /**
   * Index of element to retrieve entity ID from.
   *
   * @var int
   */
  private $path_id_index = 0;

  /**
   * Title for translation editing screen.
   *
   * @var string
   */
  private $title = '';

  /**
   * Configuration entity type name.
   *
   * @var string
   */
  private $entity_type = '';

  /**
   * Configuration name prefix for the entity type.
   *
   * @var string
   */
  private $config_prefix = '';

  /**
   * Menu item type.
   *
   * @var int
   */
  private $menu_type = MENU_LOCAL_TASK;

  /**
   * Indicator whether an edit tab should be added.
   *
   * @var bool
   */
  private $add_edit_tab = FALSE;

  /**
   * Constructor for configuration group.
   *
   * @param string $base_path
   *   Base path to attach the group user interface to.
   * @param int $path_id_index
   *   Index of element to retrieve entity ID from.
   * @param string $entity_type
   *   Configuration entity type name.
   * @param string $title
   *   Title for translation editing screen. Use @label for entity
   *   label placement.
   * @param string $config_prefix
   *   Configuration name prefix for the entity type.
   * @param int $menu_type
   *   (optional) Menu item type to add.
   * @param bool $add_edit_tab
   *   (optional) Indicator whether an edit tab should be added. If there are
   *   no existing tabs on the page, for the translation tab to show up, an
   *   edit tab will need to be added.
   */
  function __construct($base_path, $path_id_index, $entity_type, $title, $config_prefix, $menu_type = MENU_LOCAL_TASK, $add_edit_tab = FALSE) {
    $this->base_path = $base_path;
    $this->path_id_index = $path_id_index;
    $this->entity_type = $entity_type;
    $this->title = $title;
    $this->config_prefix = $config_prefix;
    $this->menu_type = $menu_type;
    $this->add_edit_tab = $add_edit_tab;
  }

  /**
   * Implements \Drupal\config_translation\ConfigMapperInterface::hasEditTab().
   */
  public function hasEditTab() {
    return $this->add_edit_tab;
  }

  /**
   * Implements \Drupal\config_translation\ConfigMapperInterface::getBasePath().
   */
  public function getBasePath() {
    return $this->base_path;
  }

  /**
   * Implements \Drupal\config_translation\ConfigMapperInterface::getMenuType().
   */
  public function getMenuType() {
    return $this->menu_type;
  }

  /**
   * Implements \Drupal\config_translation\ConfigMapperInterface::getPathIdIndex().
   */
  public function getPathIdIndex() {
    return $this->path_id_index;
  }

  /**
   * Implements \Drupal\config_translation\ConfigMapperInterface::getConfigGroup().
   */
  public function getConfigGroup($arg = NULL) {
    // In this implementation, $arg is the config entity loaded via the menu
    // (for contact module for example) or a string (config entity ID), in
    // which case we need to load the entity (for views for example).
    if (!isset($arg)) {
      return NULL;
    }
    if (is_string($arg)) {
      $entity = entity_load($this->entity_type, $arg);
    }
    else {
      $entity = $arg;
    }

    // Replace path segment with the ID of the entity for further processing.
    $path = explode('/', $this->base_path);
    $path[$this->path_id_index] = $entity->id();
    $base_path = join('/', $path);

    // Replace entity label in template title.
    $title = format_string($this->title, array('@label' => $entity->label()));

    // The list of config IDs belonging to this entity.
    $names = array($this->config_prefix . '.' . $entity->id());

    return new ConfigGroupMapper($base_path, $title, $names, $this->menu_type, $this->add_edit_tab);
  }

}
