<?php

/**
 * @file
 * Contains \Drupal\config_translation\ConfigGroupMapper.
 */

namespace Drupal\config_translation;

use Drupal\Core\Language\Language;

/**
 * Configuration editing screen mapper.
 */
class ConfigGroupMapper implements ConfigMapperInterface {

  /**
   * Base path for group to attach to.
   *
   * @var string
   */
  private $base_path = '';

  /**
   * Title for translation editing screen.
   *
   * @var string
   */
  private $title = '';

  /**
   * List of configuration names.
   *
   * @var array
   */
  private $names = array();

  /**
   * Menu item type.
   *
   * @var int
   */
  private $menu_type = MENU_LOCAL_TASK;

  /**
   * Indicator whether an edit tab should be added.
   *
   * @var bool
   */
  private $add_edit_tab = FALSE;

  /**
   * Constructor for configuration group.
   *
   * @param string $base_path
   *   Base path to attach the group user interface to.
   * @param string $title
   *   Title for translation editing screen.
   * @param array $names
   *   List of configuration names in group.
   * @param int $menu_type
   *   (optional) Menu item type to add.
   * @param bool $add_edit_tab
   *   (optional) Indicator whether an edit tab should be added. If there are
   *   no existing tabs on the page, for the translation tab to show up, an
   *   edit tab will need to be added.
   */
  function __construct($base_path, $title, $names, $menu_type = MENU_LOCAL_TASK, $add_edit_tab = FALSE) {
    $this->base_path = $base_path;
    $this->title = $title;
    $this->names = $names;
    $this->menu_type = $menu_type;
    $this->add_edit_tab = $add_edit_tab;
  }

  /**
   * Implements \Drupal\config_translation\ConfigMapperInterface::hasEditTab().
   */
  public function hasEditTab() {
    return $this->add_edit_tab;
  }

  /**
   * Implements \Drupal\config_translation\ConfigMapperInterface::getBasePath().
   */
  public function getBasePath() {
    return $this->base_path;
  }

  /**
   * Returns title of this configuration group.
   *
   * @return string
   *   The group title.
   */
  public function getTitle() {
    return $this->title;
  }

  /**
   * Returns list of names in this configuration group.
   *
   * @return array
   *   An array of configuration names in this group.
   */
  public function getNames() {
    return $this->names;
  }

  /**
   * Implements \Drupal\config_translation\ConfigMapperInterface::getMenuType().
   */
  public function getMenuType() {
    return $this->menu_type;
  }

  /**
   * Implements \Drupal\config_translation\ConfigMapperInterface::getPathIdIndex().
   */
  public function getPathIdIndex() {
    return NULL;
  }

  /**
   * Implements \Drupal\config_translation\ConfigMapperInterface::getConfigGroup().
   */
  public function getConfigGroup($arg = NULL) {
    // This is already a config group.
    return $this;
  }

  /**
   * Returns the original language code of the configuration group.
   *
   * @todo
   *   Assumes the langcode of the first element for now. Might lead to
   *   inconsistency. Revisit.
   */
  public function getLangcode() {
    return config_translation_original_langcode($this->names[0]);
  }

  /**
   * Returns an array with configuration data for the group.
   *
   * @return array
   *   Configuration data keyed by configuration names in the group.
   */
  public function getConfigData() {
    $config_data = array();
    foreach ($this->names as $name) {
      $config_data[$name] = config($name)->get();
    }
    return $config_data;
  }

  /**
   * Checks that all pieces of this configuration group has schema.
   *
   * @return bool
   *   TRUE if all of the group elements have schema, FALSE otherwise.
   */
  public function hasSchema() {
    foreach ($this->names as $name) {
      if (!config_translation_has_schema($name)) {
        return FALSE;
      }
    }
    return TRUE;
  }

  /**
   * Checks that all pieces of this configuration group has translatables.
   *
   * @return bool
   *   TRUE if all of the group elements have translatables, FALSE otherwise.
   */
  public function hasTranslatable() {
    foreach ($this->names as $name) {
      if (!config_translation_has_translatable($name)) {
        return FALSE;
      }
    }
    return TRUE;
  }

  /**
   * Checks whether there is already a translation for this group.
   *
   * @param \Drupal\Core\Language\Language $language
   *   A language object.
   *
   * @return bool
   *   TRUE if any of the group elements have a translation in the given
   *   language, FALSE otherwise.
   */
  public function hasTranslation(Language $language) {
    foreach ($this->names as $name) {
      if (config_translation_exists($name, $language)) {
        return TRUE;
      }
    }
    return FALSE;
  }

  /**
   * Adds the given configuration name to the list of names in the group.
   *
   * @param string $name
   *   Configuration name.
   */
  public function addName($name) {
    $this->names[] = $name;
  }

}
