<?php

/**
 * @file
 * Definition of \Drupal\config_translation\Tests\ConfigTranslationUITest.
 */

namespace Drupal\config_translation\Tests;

use Drupal\Core\Config\FileStorage;
use Drupal\simpletest\WebTestBase;

/**
 * Functional tests for the Language list configuration forms.
 */
class ConfigTranslationUITest extends WebTestBase {

  /**
   * Modules to enable.
   *
   * @var array
   */
  public static $modules = array('contact', 'config_translation');

  /**
   * Languages to enable.
   *
   * @var array
   */
  protected $langcodes = array('fr', 'ta');

  public static function getInfo() {
    return array(
      'name' => 'Configuration translation',
      'description' => 'Translate settings and entities to various languages',
      'group' => 'Configuration translation',
    );
  }

  function setUp() {
    parent::setUp();
    $permissions = array(
      'translate configuration',
      'administer languages',
      'administer site configuration',
      'administer contact forms',
      'access site-wide contact form'
    );
    // Create and login user.
    $admin_user = $this->drupalCreateUser($permissions);
    $this->drupalLogin($admin_user);

    // Add languages.
    foreach ($this->langcodes as $langcode) {
      $edit = array(
        'predefined_langcode' => $langcode,
      );
      $this->drupalPost('admin/config/regional/language/add', $edit, t('Add language'));
    }

  }

  /**
   * Test case for site information translation interface.
   */
  function testSiteInformationTranslationUI() {
    $site_name = 'Site name for testing configuration translation';
    $site_slogan = 'Site slogan for testing configuration translation';
    $translation_base_url = 'admin/config/system/site-information/translate';

    // Set site name and slogan for default language.
    $this->setSiteInformation($site_name, $site_slogan);

    // Check translation tab exist.
    $this->assertLinkByHref($translation_base_url);

    $this->drupalGet($translation_base_url);

    // Check 'Add' link of French to visit add page.
    $this->assertLinkByHref($translation_base_url . '/add/fr');
    $this->clickLink(t('Add'));

    // Make sure original text is present on this page.
    $this->assertRaw($site_name);
    $this->assertRaw($site_slogan);

    // Update site name and slogan for French.
    $edit = array(
      '0[name]' => 'FR ' . $site_name,
      '0[slogan]' => 'FR ' . $site_slogan,
    );
    $this->drupalPost($translation_base_url . '/add/fr', $edit, t('Save translation'));
    $this->assertRaw(t('Updated French configuration translations successfully.'));

    // Check for edit, delete links (and no 'add' link) for French language.
    $this->assertNoLinkByHref($translation_base_url . '/add/fr');
    $this->assertLinkByHref($translation_base_url . '/edit/fr');
    $this->assertLinkByHref($translation_base_url . '/delete/fr');

    // Check translation saved proper.
    $this->drupalGet($translation_base_url . '/edit/fr');
    $this->assertFieldByName('0[name]', 'FR ' . $site_name);
    $this->assertFieldByName('0[slogan]', 'FR ' . $site_slogan);

    // Check French translation of site name and slogan are in place.
    $this->drupalGet('fr');
    $this->assertRaw('FR ' . $site_name);
    $this->assertRaw('FR ' . $site_slogan);
  }

  /**
   * Test case for site information translation interface.
   */
  function testSourceValueDuplicateSave() {
    $site_name = 'Site name for testing configuration translation';
    $site_slogan = 'Site slogan for testing configuration translation';
    $translation_base_url = 'admin/config/system/site-information/translate';
    $this->setSiteInformation($site_name, $site_slogan);

    $this->drupalGet($translation_base_url);

    // Case 1: Update new value for site slogan and site name.
    $edit = array(
      '0[name]' => 'FR ' . $site_name,
      '0[slogan]' => 'FR ' . $site_slogan,
    );
    // First time, no overrides, so just Add link.
    $this->drupalPost($translation_base_url . '/add/fr', $edit, t('Save translation'));

    // Read overridden file from active config.
    $file_storage = new FileStorage($this->configDirectories[CONFIG_ACTIVE_DIRECTORY]);
    $config_parsed = $file_storage->read('locale.config.fr.system.site');

    // Expect both name and slogan in language specific file.
    $expected = array(
      'name' => 'FR ' . $site_name,
      'slogan' => 'FR ' . $site_slogan
    );
    $this->assertEqual($expected, $config_parsed);

    // Case 2: Update new value for site slogan and default value for site name.
    $edit = array(
      '0[name]' => $site_name,
      '0[slogan]' => 'FR ' . $site_slogan,
    );
    $this->drupalPost($translation_base_url . '/edit/fr', $edit, t('Save translation'));
    $config_parsed = $file_storage->read('locale.config.fr.system.site');

    // Expect only slogan in language specific file.
    $expected = array(
      'slogan' => 'FR ' . $site_slogan
    );
    $this->assertEqual($expected, $config_parsed);

    // Case 3: Keep default value for site name and slogan.
    $edit = array(
      '0[name]' => $site_name,
      '0[slogan]' => $site_slogan,
    );
    $this->drupalPost($translation_base_url . '/edit/fr', $edit, t('Save translation'));
    $config_parsed = $file_storage->read('locale.config.fr.system.site');

    // Expect no language specific file.
    $this->assertFalse($config_parsed);
  }

  /**
   * Test case for contact category translation.
   */
  function testContactConfigEntityTranslation() {
    $this->drupalGet('admin/structure/contact');

    // Check for default contact form config entity from Contact module.
    $this->assertLinkByHref('admin/structure/contact/manage/feedback/edit');

    // Save default language configuration.
    $edit = array(
      'label' => 'Send your feedback',
      'recipients' => 'sales@example.com,support@example.com',
      'reply' => 'Thank you for your mail'
    );
    $this->drupalPost('admin/structure/contact/manage/feedback/edit', $edit, t('Save'));

    // Visit the form to confirm the changes.
    $this->drupalGet('contact/feedback');
    $this->assertText('Send your feedback');

    foreach ($this->langcodes as $langcode) {
      // Update translatable fields.
      $edit = array(
        '0[label]' => 'Website feedback - ' . $langcode,
        '0[reply]' => 'Thank you for your mail - ' . $langcode
      );

      // Save language specific version of form.
      $this->drupalPost('admin/structure/contact/manage/feedback/translate/add/' . $langcode, $edit, t('Save translation'));

      // Visit language specific version of form to check label exists.
      $this->drupalGet($langcode . '/contact/feedback');
      $this->assertText('Website feedback - ' . $langcode);

      // Submit feedback.
      $edit = array(
        'subject' => 'Test subject',
        'message' => 'Test message'
      );
      $this->drupalPost(NULL, $edit, t('Send message'));
    }

    // We get all emails so no need to check inside the loop.
    $captured_emails = $this->drupalGetMails();

    // Check language specific auto reply text in email body.
    foreach ($captured_emails as $email) {
      if ($email['id'] == 'contact_page_autoreply') {
        // Trim because we get an added newline for the body.
        $this->assertEqual(trim($email['body']), 'Thank you for your mail - ' . $email['langcode']);
      }
    }

  }

  /**
   * Test case for account settings translation interface.
   *
   * This is the only special case so far where we have multiple config names
   * involved building up one configuration translation form. Test that the
   * translations are saved for all configuration names properly.
   */
  function testAccountSettingsConfigurationTranslation() {
    $this->drupalGet('admin/config/people/accounts/translate');
    $this->assertLinkByHref('admin/config/people/accounts/translate/add/fr');

    // Update account settings fields for French.
    $edit = array(
      '0[anonymous]' => 'Anonyme',
      '1[status_blocked][status_blocked.subject]' => 'Testing, your account is blocked.',
      '1[status_blocked][status_blocked.body]' => 'Testing account blocked body.'
    );

    $this->drupalPost('admin/config/people/accounts/translate/add/fr', $edit, t('Save translation'));

    // Make sure the changes are saved and loaded back properly.
    $this->drupalGet('admin/config/people/accounts/translate/edit/fr');
    foreach ($edit as $value) {
      $this->assertRaw($value);
    }
  }

  /**
   * Helper to set site name and slogan for default language.
   *
   * @param string $site_name
   * @param string $site_slogan
   */
  protected function setSiteInformation($site_name, $site_slogan) {
    $edit = array(
      'site_name' => $site_name,
      'site_slogan' => $site_slogan
    );
    $this->drupalPost('admin/config/system/site-information', $edit, t('Save configuration'));
    $this->assertRaw(t('The configuration options have been saved.'));

  }
}
